@@ -70,17 +70,25 @@ const char *libarchiver_mime_types[] = {
 		"application/vnd.ms-cab-compressed",
 		"application/x-7z-compressed",
 		"application/x-ar",
+		"application/x-bzip-compressed-tar",
+		"application/x-cbr",
+		"application/x-cbz",
 		"application/x-cd-image",
 		"application/x-compressed-tar",
 		"application/x-cpio",
-		"application/x-bzip-compressed-tar",
+		"application/x-deb",
 		"application/x-lha",
+		"application/x-lrzip-compressed-tar",
 		"application/x-lzip-compressed-tar",
 		"application/x-lzma-compressed-tar",
+		"application/x-lzop-compressed-tar",
+		"application/x-rar",
+		"application/x-rpm",
 		"application/x-tar",
 		"application/x-tarz",
 		"application/x-xar",
 		"application/x-xz-compressed-tar",
+		"application/zip",
 		NULL };
 
 
@@ -113,11 +121,37 @@ fr_archive_libarchive_get_capabilities (FrArchive  *archive,
 		return capabilities;
 	}
 
+	/* give priority to 7z, unzip and zip that supports ZIP files better. */
+	if ((strcmp (mime_type, "application/zip") == 0)
+	    || (strcmp (mime_type, "application/x-cbz") == 0))
+	{
+		if (_g_program_is_available ("7z", check_command)) {
+			return capabilities;
+		}
+		if (!_g_program_is_available ("unzip", check_command)) {
+			capabilities |= FR_ARCHIVE_CAN_READ;
+		}
+		if (!_g_program_is_available ("zip", check_command)) {
+			capabilities |= FR_ARCHIVE_CAN_WRITE;
+		}
+		return capabilities;
+	}
+
+	/* tar.lrz format requires external lrzip */
+	if (strcmp (mime_type, "application/x-lrzip-compressed-tar") == 0) {
+		if (!_g_program_is_available ("lrzip", check_command))
+			return capabilities;
+	}
+
 	capabilities |= FR_ARCHIVE_CAN_READ;
 
 	/* read-only formats */
 	if ((strcmp (mime_type, "application/vnd.ms-cab-compressed") == 0)
+	    || (strcmp (mime_type, "application/x-cbr") == 0)
+	    || (strcmp (mime_type, "application/x-deb") == 0)
 	    || (strcmp (mime_type, "application/x-lha") == 0)
+	    || (strcmp (mime_type, "application/x-rar") == 0)
+	    || (strcmp (mime_type, "application/x-rpm") == 0)
 	    || (strcmp (mime_type, "application/x-xar") == 0))
 	{
 		return capabilities;
@@ -1049,6 +1083,10 @@ _archive_write_set_format_from_context (struct archive *a,
 		archive_write_set_format_pax_restricted (a);
 		archive_filter = ARCHIVE_FILTER_GZIP;
 	}
+	else if (_g_str_equal (mime_type, "application/x-lrzip-compressed-tar")) {
+		archive_write_set_format_pax_restricted (a);
+		archive_filter = ARCHIVE_FILTER_LRZIP;
+	}
 	else if (_g_str_equal (mime_type, "application/x-lzip-compressed-tar")) {
 		archive_write_set_format_pax_restricted (a);
 		archive_filter = ARCHIVE_FILTER_LZIP;
@@ -1057,6 +1095,10 @@ _archive_write_set_format_from_context (struct archive *a,
 		archive_write_set_format_pax_restricted (a);
 		archive_filter = ARCHIVE_FILTER_LZMA;
 	}
+	else if (_g_str_equal (mime_type, "application/x-lzop-compressed-tar")) {
+		archive_write_set_format_pax_restricted (a);
+		archive_filter = ARCHIVE_FILTER_LZOP;
+	}
 	else if (_g_str_equal (mime_type, "application/x-xz-compressed-tar")) {
 		archive_write_set_format_pax_restricted (a);
 		archive_filter = ARCHIVE_FILTER_XZ;
@@ -1080,6 +1122,10 @@ _archive_write_set_format_from_context (struct archive *a,
 	else if (_g_str_equal (mime_type, "application/x-7z-compressed")) {
 		archive_write_set_format_7zip (a);
 	}
+	else if (_g_str_equal (mime_type, "application/zip")
+	    || _g_str_equal (mime_type, "application/x-cbz")) {
+		archive_write_set_format_zip (a);
+	}
 
 	/* set the filter */
 
@@ -1096,12 +1142,18 @@ _archive_write_set_format_from_context (struct archive *a,
 		case ARCHIVE_FILTER_GZIP:
 			archive_write_add_filter_gzip (a);
 			break;
+		case ARCHIVE_FILTER_LRZIP:
+			archive_write_add_filter_lrzip (a);
+			break;
 		case ARCHIVE_FILTER_LZIP:
 			archive_write_add_filter_lzip (a);
 			break;
 		case ARCHIVE_FILTER_LZMA:
 			archive_write_add_filter_lzma (a);
 			break;
+		case ARCHIVE_FILTER_LZOP:
+			archive_write_add_filter_lzop (a);
+			break;
 		case ARCHIVE_FILTER_XZ:
 			archive_write_add_filter_xz (a);
 			break;
@@ -1641,6 +1693,7 @@ fr_archive_libarchive_add_files (FrArchive           *archive,
 
 typedef struct {
 	GHashTable *files_to_remove;
+	gboolean    remove_all_files;
 	int         n_files_to_remove;
 } RemoveData;
 
@@ -1648,7 +1701,8 @@ typedef struct {
 static void
 remove_data_free (RemoveData *remove_data)
 {
-	g_hash_table_unref (remove_data->files_to_remove);
+	if (remove_data->files_to_remove != NULL)
+		g_hash_table_unref (remove_data->files_to_remove);
 	g_free (remove_data);
 }
 
@@ -1662,7 +1716,7 @@ _remove_files_begin (SaveData *save_data,
 
 	fr_archive_progress_set_total_files (load_data->archive, remove_data->n_files_to_remove);
 	fr_archive_progress_set_total_bytes (load_data->archive,
-				FR_ARCHIVE_LIBARCHIVE (load_data->archive)->priv->uncompressed_size);
+					     FR_ARCHIVE_LIBARCHIVE (load_data->archive)->priv->uncompressed_size);
 }
 
 
@@ -1676,6 +1730,9 @@ _remove_files_entry_action (SaveData             *save_data,
 	WriteAction  action;
 	const char  *pathname;
 
+	if (remove_data->remove_all_files)
+		return WRITE_ACTION_SKIP_ENTRY;
+
 	action = WRITE_ACTION_WRITE_ENTRY;
 	pathname = archive_entry_pathname (w_entry);
 	if (g_hash_table_lookup (remove_data->files_to_remove, pathname) != NULL) {
@@ -1701,12 +1758,17 @@ fr_archive_libarchive_remove_files (FrArchive           *archive,
 	GList      *scan;
 
 	remove_data = g_new0 (RemoveData, 1);
-	remove_data->files_to_remove = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-	remove_data->n_files_to_remove = 0;
-	for (scan = file_list; scan; scan = scan->next) {
-		g_hash_table_insert (remove_data->files_to_remove, g_strdup (scan->data), GINT_TO_POINTER (1));
-		remove_data->n_files_to_remove++;
+	remove_data->remove_all_files = (file_list == NULL);
+	if (! remove_data->remove_all_files) {
+		remove_data->files_to_remove = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+		remove_data->n_files_to_remove = 0;
+		for (scan = file_list; scan; scan = scan->next) {
+			g_hash_table_insert (remove_data->files_to_remove, g_strdup (scan->data), GINT_TO_POINTER (1));
+			remove_data->n_files_to_remove++;
+		}
 	}
+	else
+		remove_data->n_files_to_remove = archive->files->len;
 
 	_fr_archive_libarchive_save (archive,
 				     FALSE,
